/** This is vorbisfile_example.c (that comes with the ogg vorbis sdk from xiph.org)
 *  translated to D for use with Derelict by Eric Poggel (JoeCoder). 
 *  It has been modified to load an ogg vorbis file as the first argument, decode
 *  it, and save it to the filename specified in the second argument.  You can
 *  Use a program like Audacity (Project->Import Raw Data) to play back the raw data.
 *  It writes it out as 16-bit and the same number of channels as the input source.*/

import std.c.stdio;
import std.file;
import derelict.ogg.vorbisfile;
import derelict.ogg.vorbistypes;

// take 4k out of the data segment, not the stack
byte pcmout[4096];	// usually the max ov_read will decode at a time?

void main(char[][] args)
{	printf("Decoding '%.*s' to '%.*s'\n", args[1], args[2]);

	// Init
	DerelictVorbisFile.load();

	OggVorbis_File vf;
	int eof=0;
	int current_section;
	std.file.write(args[2], null); // erase file that we'll output to

	// Open (This calls a version of ov_open rewritten in D)
	FILE *a = fopen(args[1], "rb");
	if(ov_open(a, &vf, null, 0) < 0)
	{	printf("Input does not appear to be an Ogg bitstream.\n");
		return;
	}

	// Throw the comments plus a few lines about the bitstream we're decoding
	char **ptr 	= ov_comment(&vf, -1).user_comments;
	vorbis_info *vi = ov_info(&vf, -1);
	while(*ptr)
	{	fprintf(stderr,"%s\n",*ptr);
		++ptr;
	}
	printf("\nBitstream is %d channel, %ldHz\n", vi.channels, vi.rate);
	printf("\nDecoded length: %ld samples\n", cast(long)ov_pcm_total(&vf, -1));
	printf("Encoded by: %s\n\n",ov_comment(&vf, -1).vendor);

	// The decoding loop
	printf("Starting\n\n");
	while(!eof)
	{	long ret = ov_read(&vf, pcmout, pcmout.sizeof, 0, 2, 1, &current_section);
		if (ret == 0)
			eof=1;
		else if (ret<0)
		{	// Error in the stream.  Not a problem, just reporting it in case someone cares.
			switch(ret)
			{	case OV_EREAD: printf("OV_EREAD\n"); break;
				case OV_ENOTVORBIS: printf("OV_ENOTVORBIS\n"); break;
				case OV_EVERSION: printf("OV_EVERSION\n"); break;
				case OV_EBADHEADER: printf("OV_EBADHEADER\n"); break;
				case OV_EFAULT: printf("OV_EFAULT\n"); break;
			}
		}else
		{	// we don't bother dealing with sample rate changes, etc, but you'll have to
			std.file.append("output.raw", pcmout[0..ret]);
		}
	}

	// cleanup
	ov_clear(&vf);
	printf("Done.\n");
}
